indexing

	description: "Various routines to help creating XML."

	author: "Berend de Boer"
	date: "$Date: 2003/10/02 $"
	revision: "$Revision: #2 $"


class

	EPX_XML_ROUTINES


inherit

	XM_UNICODE_CHARACTER_CLASSES


feature -- Queries

	is_valid_attribute_name (attribute: STRING): BOOLEAN is
			-- Is `attribute' a valid XML attribute name?
		local
			first,
			other: CHARACTER
			i: INTEGER
		do
			Result := attribute /= Void and then not attribute.is_empty
			if Result then
				first := attribute.item (1)
				Result := ValidFirstChars.has (first)
				from
					i := 2
				until
					not Result or else i >= attribute.count
				loop
					other := attribute.item (i)
					Result := ValidOtherChars.has (other)
					i := i + 1
				end
			end
		end

	is_valid_ncname (tag: STRING): BOOLEAN is
			-- Is `tag' a valid XML NCName (no colons)?
		do
			Result :=
				is_valid_tag_name (tag) and then
				not tag.has (':')
		ensure
			valid_result:
				Result implies
					is_valid_tag_name (tag) and then
					not tag.has (':')
		end

	is_valid_tag_name (tag: STRING): BOOLEAN is
			-- Is `tag' a valid XML tag name?
		local
			i: INTEGER
		do
			Result :=
				tag /= Void and then
				not tag.is_empty and then
				is_name_first (tag.item_code (1))
			if Result then
				from
					i := 2
				variant
					(tag.count + 1) - i
				until
					not Result or else
					i > tag.count
				loop
					Result := is_name_char (tag.item_code (i))
					i := i + 1
				end
			end
		end

	has_invalid_control_characters (s: STRING): BOOLEAN is
			-- If s is not Void, does `s' contain characters in the range
			-- 0x00-0x1F other then TAB (0x09), CR (0x0A) and LF (0x0D)?
		local
			i: INTEGER
			c: CHARACTER
		do
			if s /= Void then
				from
					Result := False
					i := 1
				until
					Result or else
					i > s.count
				loop
					c := s.item (i)
					if c.code < 32 then
						Result :=
							c.code /= 10 and then
							c.code /= 13 and then
							c.code /= 9
					end
					i := i + 1
				end
			end
		end


feature -- Conversions

	as_valid_tag_name (a_tag_name: STRING): STRING is
			-- Return `a_tag_name' with invalid characters removed or
			-- replaced. Currently it only replaces characters judged
			-- invalid by `is_valid_tag_name' by '-'.
		require
			a_tag_name_not_empty: a_tag_name /= Void and then not a_tag_name.is_empty
		local
			i: INTEGER
		do
			if not is_name_first (a_tag_name.item_code (1)) then
				Result := clone (a_tag_name)
				Result.put ('_', 1)
			end
			from
				i := 2
			invariant
				Result = Void or else Result.count = a_tag_name.count
			variant
				(a_tag_name.count + 1) - i
			until
				i > a_tag_name.count
			loop
				if not is_name_char (a_tag_name.item_code (i)) then
					if Result = Void then
						Result := clone (a_tag_name)
					end
					Result.put ('-', i)
				end
				i := i + 1
			end
			if Result = Void then
				Result := a_tag_name
			end
		ensure
			valid_tag_name: is_valid_tag_name (Result)
		end

	quote_apostrophe (s: STRING): STRING is
			-- Replace every occurrence of ' by &apos;
			-- If `s' is Void, it returns Void.
		require
			s_not_void: s /= Void
		local
			p: INTEGER
			i: INTEGER
			c: CHARACTER
		do
			-- Optimized for the case that `s' does not contain an
			-- apostrophe. Read optimized as: I think it is faster.
			p := s.index_of ('%'', 1)
			if p = 0 then
				Result := s
			else
				from
					create Result.make_from_string (s)
					i := p
				until
					i > Result.count
				loop
					c := Result.item (i)
					if c = '%'' then
						Result.put ('&', i)
						Result.insert_string (PartQuoteApos, i + 1)
						i := i + QuoteApos.count
					else
						i := i + 1
					end
				end
			end
		ensure
			result_not_void: Result /= Void
			result_at_least_s: Result.count >= s.count
			has_no_apostrophe: not Result.has ('%'')
		end


feature {NONE} -- Data sanitize

	Ampersand_code: INTEGER is 38
			-- '&'

	QuoteAmp: STRING is "&amp;"
	QuoteApos: STRING is "&apos;"
	QuoteLt: STRING is "&lt;"
	QuoteGt: STRING is "&gt;"
	QuoteQuote: STRING is "&quot;"

	PartQuoteAmp: STRING is "amp;"
	PartQuoteApos: STRING is "apos;"
	PartQuoteLt: STRING is "lt;"
	PartQuoteGt: STRING is "gt;"
	PartQuoteQuote: STRING is "quot;"


feature -- Constants from the XML specification, should be Unicode...

	ValidFirstChars: STRING is "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_:"
			-- Which characters are valid as the first character.

	ValidOtherChars: STRING is "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_:.-0123456789"
			-- Which characters are valid as second etc characters.


end
